/* ==============================================================
 * LicqRmsFrontend.java, created on 1 oct. 2003 by paul
 * --------------------------------------------------------------
 *
 * Copyright  Paul Pinault, Disk91 2003.
 *
 * Tout dtenteur d'une copie de ce programme et des fichiers de
 * documentation associs (le  logiciel ) bnficie des droits
 * d'exploitation, de copie, de modification, de rutilisation, de
 * publication, de distribution gratuite ou commerciale, et reste libre
 * de distribuer l'ensemble et/ou d'en vendre des copies en octroyant ces
 * mmes droits aux utilisateurs sous rserve de signaler son utilisation
 *  son auteur par mail, indiquant votre identit et le nom de votre
 * projet,  l'adresse suivante : dpl@disk91.com, ce signalement implique
 * de votre part un accord d'utilisation de ces informations dans le but de
 * promouvoir le dit logiciel. En outre, les droit sont accords sous
 * rserve d'inclure le copyright ci-dessus et l'avertissement suivant dans
 * toute copie, totale ou partielle, du programme :
 *
 * CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
 * QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
 * COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
 * EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
 * DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
 * TOUT OU PARTIE DE CE PROGRAMME.
 *
 * Les noms Paul Pianult et  Disk91 ne saurait tre utilis dans le but
 * de promouvoir ou de lgitimer un produit driv de ce programme sans
 * autorisation crite pralable  cette effet.
 *
 * 
 * =============================================================
 */
package disk.icq;

import java.io.*;
import java.net.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * =============================================================================
 * disk91.icq::LicqRmsFrontend.java
 * ------------------------------------------------------------------------------------------------
 * Classe FrontEnd pour la connexion au service RMS de Licq (controle de licq via
 * telnet)
 * =============================================================================
 */
public class LicqRmsFrontend implements javax.servlet.http.HttpSessionBindingListener {

	/** STATUS ... */
	public static final int STATUS_ONLINE 	= 1;
	public static final int STATUS_OFFLINE	= 2;
	public static final int STATUS_AWAY	= 3;
	public static final int STATUS_NA	= 4;
	public static final int STATUS_DND	= 5;
	public static final int STATUS_FFC	= 6;
	public static final int STATUS_OCCUP	= 7;
	public static final int STATUS_UNKNOW  	= 0;
	public static final int STATUS_ERROR	= -1;

	/** Informations de connexion */
	
	/** Port de connexion */
	private int				rmsPort;
	/** Adresse du serveur*/
	private String				server;
	/** UIN pour le login */
	private String  			uin;

	/** Acces en ecriture avec le driver */
	private PrintWriter   			pw;
	/** Lecture du flux */
	private BufferedReader 			br;
	/** Connexion */
	private Socket         			socket;
	
	/** Status */
	private boolean 			connected;
	private boolean 			online;
	private int				currentStatus;

	/** -------------- Utilisateurs ------------ */
	private HashMap 			users;
	private boolean				listAll;
	

	
   /**
    * Constructeur : LicqRmsFrontend
    */
	public LicqRmsFrontend() {
		init();
	}
	
	/**
	 * Constructeur : LicqRmsFrontend
	 * 
	 * @param rmsPort - Port TCP de connexion avec le serveur rms
	 * @param server - Nom de la machine sur laquelle est le serveur
	 */
	public LicqRmsFrontend(int rmsPort, String server) {
		this.rmsPort 	= rmsPort;
		this.server    	= server;
		init();
	}

	/**
	 * connect() - Etablie la connexion avec le serveur rms licq
	 * 
	 * @param uin - uin utilis comme login sur le serveur
	 * @param passwd - mot de passe associ a l'uin
	 * @return tru si la connexion est tablie.
	 * 
	 * @author paul
	 * @version 1.0
	 */
	public boolean connect(String uin, String passwd) {
		this.uin = uin;
		String str;
		try {
		  socket = new Socket(InetAddress.getByName(this.server),this.rmsPort);

		  pw  = new PrintWriter(
		            new BufferedWriter (	 
		                new OutputStreamWriter (
					socket.getOutputStream())),true);
		  br  = new BufferedReader(
			    new InputStreamReader (
		 	        socket.getInputStream()));

 		  str = br.readLine(); //System.err.println(str);
		  str = br.readLine(); //System.err.println(str);
		  pw.println(this.uin);
		  str = br.readLine(); //System.err.println(str);
		  pw.println(passwd);
		  str = br.readLine(); //System.err.println(str);

		  this.connected = true;		  
	   	  return true;

		} catch ( Exception e ) {
		   return false;
		}
		
	}
	

	/**
	 * Ferme la connexion
	 * 
	 * @author paul
	 * @version 1.0
	 */
	public void term() {
		try {
			if ( pw != null && br != null ) {
				pw.println("TERM");
				this.init();
				pw.close();
				br.close();
			}			
			socket.close();
		} catch ( Exception e ) {}
	}

	/**
	 * Ferme la connexion
	 * 
	 * @author paul
	 * @version 1.0
	 */
	public void close() {
		try {
			if ( pw != null && br != null ) {
				this.setStatus(LicqRmsFrontend.STATUS_OFFLINE);
				pw.println("QUIT");
				this.init();
				pw.close();
				br.close();
			}			
			socket.close();
		} catch ( Exception e ) {}
	}
	
	// =========================== SUPPORT DU STATUS ==================================
	
	/**
	 * Recupere le status actuel de l'utilisateur ICQ
	 */
	public int getStatus() {
		if ( !this.connected ) return (STATUS_ERROR);
		flushInput();
		
		pw.println("STATUS");
		int rstatus=STATUS_ERROR;
		try {
			String status = br.readLine();
			// Si la connexion est down, le status renvoie null ... c'est un moyen de verifier cela... 
			if ( status == null ) {
				this.close();
				return STATUS_ERROR;
			}
			java.util.StringTokenizer sto = new java.util.StringTokenizer(status);
			if (sto.countTokens() >= 2) {
				if ( sto.nextToken().compareTo("202") == 0 ) {
					rstatus = this.stringToStatus(sto.nextToken());
					this.currentStatus = rstatus;
				}
			}
		} catch ( IOException e ){}
		return rstatus;		
	}
	
	/**
	 * Modifie le status de l'utilisateur
	 * @param status - Id du status choisit, parmi les constantes de status prdfinies
	 * @return true - si ce status a pu etre activ.
	 * 
	 * @author paul
	 * @version 1.0
	 */
	public boolean setStatus(int status) {
		if ( !this.connected ) return false;
		flushInput();
		
		if ( status == this.currentStatus ) return true;
		
		String s_status = null;
		switch(status) {
			case STATUS_OCCUP	: s_status ="occupied"; break;
			case STATUS_ONLINE	: s_status ="online"; break;
			case STATUS_OFFLINE	: s_status ="offline"; break;
			case STATUS_DND			: s_status ="dnd"; break;
			case STATUS_NA 			: s_status ="na"; break;
			case STATUS_FFC			: s_status ="ffc"; break;
			case STATUS_AWAY		: s_status ="away"; break;
		}
		
		if ( s_status != null && (this.online  || status == STATUS_ONLINE)) {
			pw.println("STATUS "+s_status);
			try {
				String ret = br.readLine();
				java.util.StringTokenizer sto = new java.util.StringTokenizer(ret);
				if (sto.countTokens() >= 1) {
					if ( sto.nextToken().compareTo("102") == 0 ) {
						if ( status == STATUS_ONLINE )
						   this.online = true;
						else if ( status == STATUS_OFFLINE )
						   this.online = false;
						return true;
					}
				}
			} catch ( IOException e ){}
		}
		return false;
	}

	// ============================== SUPPORT DES UTILISATEURS ======================

    /**
     * Liste les utilisateurs sans changement de style
     */
	public boolean getUserList() {
		return this.getUserList(this.listAll);	
	}

	/**
	 * Liste des utilisateurs ou raffraichi leur etat.
	 * @param all - true s'il faut lister tous les utilisateurs ou seulement ceux actifs
	 * @return true si tout va bien
	 */
	public boolean getUserList(boolean all) {
		if ( this.listAll != all ) {
			// Le mode change, on vide la table
			this.flushList();
			this.listAll = all;
		}
		if ( all )
		   return getUserList("");
	    else
		  return getUserList(" online");
	}
	
	private boolean getUserList(String param) {
		if ( !this.connected || !this.online ) return false;
		flushInput();
		
		pw.println("LIST "+param);
		try {
			boolean fin = false;
			while ( !fin ) {
				String ret = br.readLine();
				java.util.StringTokenizer sto = new java.util.StringTokenizer(ret);
				if (sto.countTokens() >= 1) {
					String id = sto.nextToken(); 
					if ( id.compareTo("204") == 0 ) {
						// Entree utilisateur
						String uin = sto.nextToken();
						String pseudo = ret.substring(14,34);
						sto = new StringTokenizer(ret.substring(34));
						int  	   pendingMsg = Integer.parseInt(sto.nextToken());
						int       rstatus = this.stringToStatus(sto.nextToken());

						licqUser lu;
						lu = (licqUser)this.users.get(uin);
						if ( lu != null ) {
							// Mise a jour des infos
							lu.setPendingMsg(pendingMsg);
							lu.setStatus(rstatus);
						} else {
							lu = new licqUser(uin, pseudo, pendingMsg, rstatus);
							this.users.put(uin, lu); 
						}

					} else if ( id.compareTo("206") == 0 )
						fin = true;
				}  
			}
				
		} catch ( IOException e ){}
		return true;
	}
	
	public Iterator getUserIterator() {
		return this.users.values().iterator();
	}
	
	private void flushList() {
		this.users = null;
		this.users = new HashMap();
	}

	// =================================== ENVOIE DE MESSAGE =========================
	
	/**
	 * Envoie un message vers un UIN indiqu.
	 * 
	 * @param uin - chaine uin du destinataire
	 * @param msg - message a envoyer
	 * 
	 */
	public void sendMessage(String uin, String msg) {
		if ( !this.connected || !this.online ) return;
		flushInput();
	
		String ret;	
		try {
			pw.println("MESSAGE "+uin);
			ret = br.readLine();
			pw.println(msg);
			pw.println(".");
			ret = br.readLine();
			ret = br.readLine();
		} catch ( IOException e ){}
	}
	
	// ============================ RECEPTION DE MESSAGE ============================
	/**
	 * Recoit un message d'un UIN dtermin
	 * 
	 * @param uin - chaine uin du destinataire
	 * @return msg reu
	 * 
	 */
	public String readMessage(String uin) {
		if ( !this.connected || !this.online ) return "ERROR";
		flushInput();
	
		String msg="";
		String ret;	
		try {
			pw.println("VIEW "+uin);
			ret = br.readLine();		// 208 Message From ... ou erreur
			java.util.StringTokenizer sto = new java.util.StringTokenizer(ret);
			if (sto.countTokens() >= 1) {
				String id = sto.nextToken(); 
				if ( id.compareTo("208") == 0 ) {
					ret = br.readLine();	// 220 Sent At ...
					ret = br.readLine();   // 220 Message Start
					
					boolean fin = false;
					do {
						ret = br.readLine();	// le message !!
						if ( ret.endsWith("223 Message Complete") ) {
							ret = ret.substring(0,ret.length()-20);
							msg += "\n"+ret;
							fin = true;
						} else
							msg += "\n"+ret;
					} while ( !fin );
							
				} else return "Evenement invalide";

			} else return "Evenement invalide";
		} catch ( IOException e ){}
		return msg;
	}
	

	// ===================================== HELPER =====================================
	
	/**
	 * Vide le buffer de lecture
	 */
	private void flushInput() {
	}
	
	/**
	 * Converti une chaine de status retourn en un id de status
	 * @param status - chaine de status
	 * @return id du status
	 * 
	 * @author paul
	 * @version 1.0
	 */
	private int stringToStatus(String status) {
		int       rstatus;
		if ( status.compareTo("Occupied") == 0) 		rstatus = STATUS_OCCUP;
		else if ( status.compareTo("Online")  == 0) 	rstatus = STATUS_ONLINE;
		else if ( status.compareTo("Offline")  == 0) 	rstatus = STATUS_OFFLINE;
		else if ( status.compareTo("Do")  == 0) 		rstatus = STATUS_DND;
		else if ( status.compareTo("Not")  == 0) 		rstatus = STATUS_NA;
		else if ( status.compareTo("Free")  == 0) 		rstatus = STATUS_FFC;
		else if ( status.compareTo("Away")  == 0)		rstatus = STATUS_AWAY;
		else rstatus = STATUS_UNKNOW;
		return rstatus;
	}

	/**
	 * Init communes aux constructeurs
	 * @author paul
	 * @version 1.0
	 */
    private void init() {
		this.connected 		= false;
		this.online 		= false;
		this.users		= new HashMap();
		this.currentStatus 	= LicqRmsFrontend.STATUS_UNKNOW;
		this.listAll		= false;
    }
	
	//	=========================== GESTION DE LA SESSION ==============================		
	 public void valueUnbound(javax.servlet.http.HttpSessionBindingEvent se) {
		 this.close();
	 }
	 public void valueBound(javax.servlet.http.HttpSessionBindingEvent se) {
	 }

	/* =================================== GETTERS AND SETTERS ================================= */


    /**
     * @return
     * 
     * @author paul
     * @version 1.0
     */
    public int getRmsPort() {
        return rmsPort;
    }

    /**
     * @return
     * 
     * @author paul
     * @version 1.0
     */
    public String getServer() {
        return server;
    }

    /**
     * @param i
     * 
     * @author paul
     * @version 1.0
     */
    public void setRmsPort(int i) {
        rmsPort = i;
    }

    /**
     * @param string
     * 
     * @author paul
     * @version 1.0
     */
    public void setServer(String string) {
        server = string;
    }

    /**
     * @return
     * 
     * @author paul
     * @version 1.0
     */
    public boolean isConnected() {
        return connected;
    }

    /**
     * @return
     * 
     * @author paul
     * @version 1.0
     */
    public boolean isOnline() {
        return online;
    }

    /**
     * @return
     * 
     * @author paul
     * @version 1.0
     */
    public boolean isListAll() {
        return listAll;
    }

}
